describe 'pi/dom/css', ->
	css = require('pi/dom/css')
	element = null

	describe 'helper functions', ->
		describe 'getPrefixed()', ->
			it 'should return a prefixed property name when passed the non-prefixed version or false if not supported', ->
				prop = css.getPrefixed('transition-duration')
				if prop
					props = ['-webkit-transition-duration', '-moz-transition-duration', '-ms-transition-duration', '-o-transition-duration']
					expect(props).toContain(prop)
				else
					expect(prop).toBeFalsy()
		
		describe 'parseNumber()', ->
			it 'should return a unit property of "%" when passed a percentage value', ->
				expect(css.parseNumber('100%')[1]).toEqual('%')
			it 'should return a unit property of "px" when passed a number value with no unit', ->
				expect(css.parseNumber(100)[1]).toEqual('px')
			it 'should return a unit property of "px" when passed a numeric string value with no unit', ->
				expect(css.parseNumber('100')[1]).toEqual('px')
			it 'should return a unit property of "" when passed a string with no unit', ->
				expect(css.parseNumber('left')[1]).toEqual('')
			it 'should return a num property that is a Number when passed a number as string', ->
				expect(css.parseNumber('100px')[0]).toEqual(100)
			it 'should return the original value when passed a non-numeric string', ->
				expect(css.parseNumber('float')[0]).toEqual('float')
		
		describe 'getShorthand()', ->
			it 'should return a longhand property', ->
				expect(css.getShorthand('margin')).toEqual('margin-top')
		
		describe 'expandShorthand()', ->
			props = css.expandShorthand('margin', '10px')

			it 'should return an array of longhand properties', ->
				expect(props['margin-top']).toBeDefined()
				expect(props['margin-bottom']).toBeDefined()
				expect(props['margin-left']).toBeDefined()
				expect(props['margin-left']).toBeDefined()
	
	describe 'getStyle()', ->
		beforeEach ->
			element = document.createElement('div')
			document.body.appendChild(element)
		afterEach ->
			document.body.removeChild(element)

		it 'should return the default for an unset style', ->
			expect(css.getStyle(element, 'display')).toEqual('block')
		it 'should return the style for a set style', ->
			element.style['display'] = 'inline'
			expect(css.getStyle(element, 'display')).toEqual('inline')

	describe 'setStyle()', ->
		beforeEach ->
			element = document.createElement('div')
			document.body.appendChild(element)
		afterEach ->
			document.body.removeChild(element)

		it 'should set the correct element style', ->
			css.setStyle(element, 'float', 'left')
			expect(element.style['float']).toEqual('left')
		it 'should set the correct element style when passed a group of properties', ->
			css.setStyle(element, {'float': 'left', 'width': '100px', 'height': 100})
			expect(element.style['width']).toEqual('100px')
		it 'should set all expanded styles for a shorthand property', ->
			css.setStyle(element, 'margin', '10px')
			expect(element.style['marginTop']).toEqual('10px')
			expect(element.style['marginBottom']).toEqual('10px')
			expect(element.style['marginLeft']).toEqual('10px')
			expect(element.style['marginRight']).toEqual('10px')
		it 'should set a numeric unit for all expanded styles of a shorthand property without unit', ->
			css.setStyle(element, 'padding', 10)
			expect(element.style['paddingTop']).toEqual('10px')
			expect(element.style['paddingBottom']).toEqual('10px')
			expect(element.style['paddingLeft']).toEqual('10px')
			expect(element.style['paddingRight']).toEqual('10px')
		it 'should replace an existing style value', ->
			element.style['float'] = 'left'
			css.setStyle(element, 'float', 'right')
			expect(element.style['float']).toEqual('right')
		it 'should replace an existing style value when many styles are set', ->
			element.style['float'] = 'left'
			element.style['width'] = '100px'
			element.style['height'] = '100px'
			css.setStyle(element, 'width', '200px')
			expect(element.style['width']).toEqual('200px')
	
	describe 'opacity styles', ->
		beforeEach ->
			element = document.createElement('div')
			document.body.appendChild(element)
		afterEach ->
			document.body.removeChild(element)

		it 'should return a float when Opacity is set to a float value', ->
			css.setStyle(element, 'opacity', 0.5)
			expect(css.getStyle(element, 'opacity')).toEqual(0.5)
		it 'should return a float when Opacity is set to a string value', ->
			css.setStyle(element, 'opacity', '0.5')
			expect(css.getStyle(element, 'opacity')).toEqual(0.5)

	describe 'clearStyle()', ->
		beforeEach ->
			element = document.createElement('div')
			document.body.appendChild(element)
		afterEach ->
			document.body.removeChild(element)

		it 'should completely remove the style rule from a setStyle() call', ->
			css.setStyle(element, {'float': 'left', 'width': 100, 'height': 100})
			css.clearStyle(element, 'float')
			expect(element.getAttribute('style')).not.toContain('float')
		it 'should completely remove a prefixed style rule from a setStyle() call', ->
			css.setStyle(element, {'box-shadow': '0px', 'width': 100, 'height': 100})
			css.clearStyle(element, 'box-shadow')
			expect(element.getAttribute('style')).not.toContain('box-shadow')
		it 'should completely remove a shorthand style rule from a setStyle() call', ->
			css.setStyle(element, {'margin': '10px', 'width': 100, 'height': 100})
			css.clearStyle(element, 'margin')
			expect(element.getAttribute('style')).not.toContain('margin')
	
	describe 'csstransition', ->
		it 'should return true if platform supports css transitions, and false if it does not', ->
			expect(css.csstransitions is true or css.csstransitions is false).toBeTruthy()
