
describe('pi/dom/css', function() {
  var css, element;
  css = require('pi/dom/css');
  element = null;
  describe('helper functions', function() {
    describe('getPrefixed()', function() {
      return it('should return a prefixed property name when passed the non-prefixed version or false if not supported', function() {
        var prop, props;
        prop = css.getPrefixed('transition-duration');
        if (prop) {
          props = ['-webkit-transition-duration', '-moz-transition-duration', '-ms-transition-duration', '-o-transition-duration'];
          return expect(props).toContain(prop);
        } else {
          return expect(prop).toBeFalsy();
        }
      });
    });
    describe('parseNumber()', function() {
      it('should return a unit property of "%" when passed a percentage value', function() {
        return expect(css.parseNumber('100%')[1]).toEqual('%');
      });
      it('should return a unit property of "px" when passed a number value with no unit', function() {
        return expect(css.parseNumber(100)[1]).toEqual('px');
      });
      it('should return a unit property of "px" when passed a numeric string value with no unit', function() {
        return expect(css.parseNumber('100')[1]).toEqual('px');
      });
      it('should return a unit property of "" when passed a string with no unit', function() {
        return expect(css.parseNumber('left')[1]).toEqual('');
      });
      it('should return a num property that is a Number when passed a number as string', function() {
        return expect(css.parseNumber('100px')[0]).toEqual(100);
      });
      return it('should return the original value when passed a non-numeric string', function() {
        return expect(css.parseNumber('float')[0]).toEqual('float');
      });
    });
    describe('getShorthand()', function() {
      return it('should return a longhand property', function() {
        return expect(css.getShorthand('margin')).toEqual('margin-top');
      });
    });
    return describe('expandShorthand()', function() {
      var props;
      props = css.expandShorthand('margin', '10px');
      return it('should return an array of longhand properties', function() {
        expect(props['margin-top']).toBeDefined();
        expect(props['margin-bottom']).toBeDefined();
        expect(props['margin-left']).toBeDefined();
        return expect(props['margin-left']).toBeDefined();
      });
    });
  });
  describe('getStyle()', function() {
    beforeEach(function() {
      element = document.createElement('div');
      return document.body.appendChild(element);
    });
    afterEach(function() {
      return document.body.removeChild(element);
    });
    it('should return the default for an unset style', function() {
      return expect(css.getStyle(element, 'display')).toEqual('block');
    });
    return it('should return the style for a set style', function() {
      element.style['display'] = 'inline';
      return expect(css.getStyle(element, 'display')).toEqual('inline');
    });
  });
  describe('setStyle()', function() {
    beforeEach(function() {
      element = document.createElement('div');
      return document.body.appendChild(element);
    });
    afterEach(function() {
      return document.body.removeChild(element);
    });
    it('should set the correct element style', function() {
      css.setStyle(element, 'float', 'left');
      return expect(element.style['float']).toEqual('left');
    });
    it('should set the correct element style when passed a group of properties', function() {
      css.setStyle(element, {
        'float': 'left',
        'width': '100px',
        'height': 100
      });
      return expect(element.style['width']).toEqual('100px');
    });
    it('should set all expanded styles for a shorthand property', function() {
      css.setStyle(element, 'margin', '10px');
      expect(element.style['marginTop']).toEqual('10px');
      expect(element.style['marginBottom']).toEqual('10px');
      expect(element.style['marginLeft']).toEqual('10px');
      return expect(element.style['marginRight']).toEqual('10px');
    });
    it('should set a numeric unit for all expanded styles of a shorthand property without unit', function() {
      css.setStyle(element, 'padding', 10);
      expect(element.style['paddingTop']).toEqual('10px');
      expect(element.style['paddingBottom']).toEqual('10px');
      expect(element.style['paddingLeft']).toEqual('10px');
      return expect(element.style['paddingRight']).toEqual('10px');
    });
    it('should replace an existing style value', function() {
      element.style['float'] = 'left';
      css.setStyle(element, 'float', 'right');
      return expect(element.style['float']).toEqual('right');
    });
    return it('should replace an existing style value when many styles are set', function() {
      element.style['float'] = 'left';
      element.style['width'] = '100px';
      element.style['height'] = '100px';
      css.setStyle(element, 'width', '200px');
      return expect(element.style['width']).toEqual('200px');
    });
  });
  describe('opacity styles', function() {
    beforeEach(function() {
      element = document.createElement('div');
      return document.body.appendChild(element);
    });
    afterEach(function() {
      return document.body.removeChild(element);
    });
    it('should return a float when Opacity is set to a float value', function() {
      css.setStyle(element, 'opacity', 0.5);
      return expect(css.getStyle(element, 'opacity')).toEqual(0.5);
    });
    return it('should return a float when Opacity is set to a string value', function() {
      css.setStyle(element, 'opacity', '0.5');
      return expect(css.getStyle(element, 'opacity')).toEqual(0.5);
    });
  });
  describe('clearStyle()', function() {
    beforeEach(function() {
      element = document.createElement('div');
      return document.body.appendChild(element);
    });
    afterEach(function() {
      return document.body.removeChild(element);
    });
    it('should completely remove the style rule from a setStyle() call', function() {
      css.setStyle(element, {
        'float': 'left',
        'width': 100,
        'height': 100
      });
      css.clearStyle(element, 'float');
      return expect(element.getAttribute('style')).not.toContain('float');
    });
    it('should completely remove a prefixed style rule from a setStyle() call', function() {
      css.setStyle(element, {
        'box-shadow': '0px',
        'width': 100,
        'height': 100
      });
      css.clearStyle(element, 'box-shadow');
      return expect(element.getAttribute('style')).not.toContain('box-shadow');
    });
    return it('should completely remove a shorthand style rule from a setStyle() call', function() {
      css.setStyle(element, {
        'margin': '10px',
        'width': 100,
        'height': 100
      });
      css.clearStyle(element, 'margin');
      return expect(element.getAttribute('style')).not.toContain('margin');
    });
  });
  return describe('csstransition', function() {
    return it('should return true if platform supports css transitions, and false if it does not', function() {
      return expect(css.csstransitions === true || css.csstransitions === false).toBeTruthy();
    });
  });
});
